#include <qpe/config.h>
#include <qapplication.h>
#include <qpe/qcopenvelope_qws.h>

#include <qheader.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qpe/mimetype.h>
#include <qtimer.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qiconset.h>
#include <qstringlist.h>
#include <qmessagebox.h>

#include "PreloadMgr.h"

PreloadMgr::PreloadMgr() : QMainWindow()
{
	setCaption(tr("PreloadMgr")) ;
	setToolBarsMovable(false) ;
    tabWidget = new QTabWidget( this);

	mainTab = new QWidget( tabWidget);
	mainTabLayout = new QGridLayout( mainTab );
	mainTabLayout->setSpacing( 6 );
	mainTabLayout->setMargin( 11 );

	appsList = new QListView( mainTab);
	appsList->addColumn( tr( "Status" ) );
	appsList->addColumn( tr( "Application Name" ) ,-1);
	appsList->addColumn( tr( "Real Name" ) ,-1);
	appsList->setAllColumnsShowFocus(TRUE);
	appsList->setShowSortIndicator(TRUE);
	appsList->setSorting(AppsName);
	mainTabLayout->addMultiCellWidget( appsList, listRow, listRow, stopButtonCol, startButtonCol );

	memLabel = new QLabel(mainTab);
	mainTabLayout->addWidget( memLabel, listRow, ststButtonCol );

	stopButton = new QPushButton( mainTab );
	stopButton->setText( tr( "STOP" ) );
	mainTabLayout->addWidget( stopButton, buttonRow, stopButtonCol );

	startButton = new QPushButton( mainTab );
	startButton->setText( tr( "START" ) );
	mainTabLayout->addWidget( startButton, buttonRow, startButtonCol );

	ststButton = new QPushButton( mainTab );
	ststButton->setText( tr( "ALLSTOP" ) );
	mainTabLayout->addWidget( ststButton, buttonRow, ststButtonCol );

	tabWidget->insertTab( mainTab, tr( "Temporary" ) );

	setTab = new QTabWidget( tabWidget);

	set = new AppLnkSet(MimeType::appsFolderName());
	menuSet.setAutoDelete(TRUE);

	loadMenuTab();
	tabWidget->insertTab( setTab, tr( "Settings" ) );

	loadDesktop();

	loadCfg();

	setCentralWidget( tabWidget );

	connect( stopButton, SIGNAL( clicked() ), this, SLOT( stopClicked() ) );
	connect( startButton,SIGNAL( clicked() ), this, SLOT( startClicked() ) );
	connect( ststButton, SIGNAL( clicked() ), this, SLOT( ststClicked() ) );

//	connect( appsList,  SIGNAL( doubleClicked(QListViewItem*)),this,SLOT( listSelected(QListViewItem*)) );
//	connect( appsList,  SIGNAL( returnPressed(QListViewItem*)),this,SLOT( listSelected(QListViewItem*)) );

	QCopChannel *channel = new QCopChannel("QPE/System", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(systemMessage(const QCString&, const QByteArray&)));

	timer = new QTimer(this);
	connect (timer, SIGNAL( timeout() ) , this , SLOT(refreshInfo()) );
	timer->start(2000);
}

PreloadMgr::~PreloadMgr() 
{
	delete set;
}

void PreloadMgr::setDocument(const QString& s)
{
	QStringList arg = QStringList::split(" ",s);
	bool er = false;
	if (arg.count() == 2 ) {
		if (arg[0] == "stop" || arg[0] == "start") {
			if (arg[0] == "stop" && arg[1] == "all") {
				ststClicked();
			} else if (arg[0] == "stop" ) {
				stopApps(arg[1]);
			} else {
				startApps(arg[1]);
			}
		} else {
			er = true;
		}
	} else {
		er = true;
	}
	if (er) {
		QMessageBox::warning(this, tr("Preload Mgr Error"),tr("parameter error")) ;
	}
}

void PreloadMgr::stopApps(const QString& name)
{
	if (QCopChannel::isRegistered(QCString("QPE/Application/"+ name))) {
		QCopEnvelope e(QCString("QPE/Application/"+ name), "quitIfInvisible()");
	}
}

void PreloadMgr::startApps(const QString& name)
{
	if (! QCopChannel::isRegistered(QCString("QPE/Application/"+ name) )) {
		QCopEnvelope e(QCString("QPE/Application/"+name), "enablePreload()");
	}
}

void PreloadMgr::stopClicked()
{
	QListViewItem* item = appsList->currentItem();
	if (item ) {
		stopApps(item->text(RealName));
	}
}
void PreloadMgr::startClicked()
{
	QListViewItem* item = appsList->currentItem();
	if (item ) {
		startApps(item->text(RealName));
	}
}

void PreloadMgr::ststClicked()
{
	for (QListViewItemIterator it(appsList) ; it.current() ; ++it) {
		if (QCopChannel::isRegistered(QCString("QPE/Application/"+it.current()->text(RealName)))) {
			QCopEnvelope e(QCString("QPE/Application/"+it.current()->text(RealName)), "quitIfInvisible()");
		}
	}
}
/*
void PreloadMgr::listSelected(QListViewItem* item)
{
}
*/

void PreloadMgr::loadMenuTab()
{
	menuSet.clear();
	QStringList types = set->types();
	for ( QStringList::Iterator it=types.begin(); it!=types.end(); ++it) {
		if ( !(*it).isEmpty() ) {
			makeMenuTab(QIconSet(set->typePixmap(*it)),set->typeName(*it),*it);
		}
	}
	makeMenuTab(QIconSet(),tr("Others"),"others");
}

void PreloadMgr::makeMenuTab(const QIconSet& ic,const QString& s ,const QString& type)
{
	QWidget* t = new QWidget(setTab);
	QBoxLayout* b = new QVBoxLayout( t );
	QListView* l = new QListView(t);
	l->addColumn( tr( "Setting" ) );
	l->addColumn( tr( "Application Name" ) ,-1);
	l->addColumn( tr( "Real Name" ) ,-1);
	l->setAllColumnsShowFocus(TRUE);
	l->setShowSortIndicator(TRUE);
	l->setSorting(AllAvail);
	b->addWidget(l);
	setTab->insertTab(t,ic,s);
	MenuSet* m = new MenuSet();
	m->type = type;
	m->wid = t;
	m->lay = b;
	m->list = l;
	menuSet.append(m);
}

void PreloadMgr::loadDesktop()
{
	QStringList apl;
	{
		Config cf= Config("Launcher");
		cf.setGroup("Preload");
		apl = cf.readListEntry("Apps",',');
	}
	QList<AppLnk> aplist = set->children();
	for (QListIterator<AppLnk> it(aplist) ; it.current() ; ++it) {
		if (it.current()->type() != "Separator") {
			MenuSet* m = findTab(it.current()->type());
			QCheckListItem *item = new QCheckListItem(m->list,"",QCheckListItem::CheckBox);
			QString can = it.current()->property("CanFastload");
			if (can =="0") {
				item->setText(AllAvail,tr("Not"));
				item->setEnabled(false);
			} else if (can == "1") {
				item->setText(AllAvail,tr("Avail"));
				item->setEnabled(true);
				if (apl.findIndex(it.current()->property("Exec")) > -1) {
					item->setOn(true);
				} else {
					item->setOn(false);
				}
				item->setEnabled(false);
			} else {
				item->setText(AllAvail,tr("Possible"));
				item->setEnabled(false);
			}
			item->setText(AllAppsName,it.current()->name());
			item->setPixmap(AllAppsName,it.current()->pixmap());
			item->setText(AllRealName,it.current()->property("Exec"));
		}
	}
}

MenuSet* PreloadMgr::findTab(QString s)
{
	for (QListIterator<MenuSet> it(menuSet) ; it.current() ; ++it) {
		if ((*it)->type == s) {
			return it;
		}
	}
	return menuSet.getLast();
}

void PreloadMgr::loadCfg()
{
	delete set;
	set = new AppLnkSet(MimeType::appsFolderName());
	Config cf= Config("Launcher");
	cf.setGroup("Preload");
	QStringList apl = cf.readListEntry("Apps",',');

	appsList->clear();
	for (QStringList::Iterator it =apl.begin() ; it != apl.end() ; ++it) {
		QString rn = (*it);
		const AppLnk* app = set->findExec(rn);
		if (app != NULL) {
			QListViewItem *item = new QListViewItem(appsList);
			item->setText(RealName,rn);
			item->setText(AppsName,app->name());
			item->setPixmap(AppsName,app->pixmap());
		}
	}
	refreshInfo();
}
	
void PreloadMgr::refreshInfo()
{
	for (QListViewItemIterator it(appsList) ; it.current() ; ++it) {
		if (QCopChannel::isRegistered(QCString("QPE/Application/"+ it.current()->text(RealName))) ) {
			it.current()->setText(Status,tr("Active"));
		} else {
			it.current()->setText(Status,tr("InActive"));
		}
	}
	QFile f("/proc/meminfo");
	if ( f.open( IO_ReadOnly ) ) {
		QTextStream t( &f);
		QString dummy = t.readLine();	// title
		t >> dummy;
		int total, used, memfree, shared, buffers, cached;
		t >> total;
		total /= 1024;
		t >> used;
		used /= 1024;
		t >> memfree;
		memfree /= 1024;
		t >> shared;
		shared /= 1024;
		t >> buffers;
		buffers /= 1024;
		t >> cached;
		cached /= 1024;
		f.close();
		int realUsed = total - ( cached + memfree );
		QString disp = tr("Memory Utilization\n\nTotal  : %1 KB\n\nUsed   : %2 KB\n\nFree   : %3 KB\nCache  : %4 KB\n").arg(total,6).arg(realUsed ,6).arg(memfree , 6).arg(cached,6);
		memLabel->setText(disp);
	}
}

void PreloadMgr::systemMessage(const QCString& msg, const QByteArray& data)
{
	QDataStream stream(data, IO_ReadOnly);
	QString ap = "";
	if (msg == "preloadDone(QString)" ) {
		stream >> ap;
		bool b =false;
		for (QListViewItemIterator it(appsList) ; it.current() ; ++it) {
			if (ap == it.current()->text(RealName)) {
				b = true;
				break;
			}
		}
		if (! b) {
			loadCfg();
		}
	}
}
